# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/commands.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import value_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/ledger/api/v1/commands.proto',
  package='com.daml.ledger.api.v1',
  syntax='proto3',
  serialized_options=b'\n\026com.daml.ledger.api.v1B\022CommandsOuterClass\252\002\026Com.Daml.Ledger.Api.V1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n%com/daml/ledger/api/v1/commands.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\"com/daml/ledger/api/v1/value.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"\xf5\x02\n\x08\x43ommands\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x13\n\x0bworkflow_id\x18\x02 \x01(\t\x12\x16\n\x0e\x61pplication_id\x18\x03 \x01(\t\x12\x12\n\ncommand_id\x18\x04 \x01(\t\x12\r\n\x05party\x18\x05 \x01(\t\x12\x31\n\x08\x63ommands\x18\x08 \x03(\x0b\x32\x1f.com.daml.ledger.api.v1.Command\x12\x35\n\x12\x64\x65\x64uplication_time\x18\t \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x37\n\x13min_ledger_time_abs\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x36\n\x13min_ledger_time_rel\x18\x0b \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x0e\n\x06\x61\x63t_as\x18\x0c \x03(\t\x12\x0f\n\x07read_as\x18\r \x03(\tJ\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08\"\xa0\x02\n\x07\x43ommand\x12\x37\n\x06\x63reate\x18\x01 \x01(\x0b\x32%.com.daml.ledger.api.v1.CreateCommandH\x00\x12;\n\x08\x65xercise\x18\x02 \x01(\x0b\x32\'.com.daml.ledger.api.v1.ExerciseCommandH\x00\x12\x45\n\rexerciseByKey\x18\x04 \x01(\x0b\x32,.com.daml.ledger.api.v1.ExerciseByKeyCommandH\x00\x12M\n\x11\x63reateAndExercise\x18\x03 \x01(\x0b\x32\x30.com.daml.ledger.api.v1.CreateAndExerciseCommandH\x00\x42\t\n\x07\x63ommand\"\x82\x01\n\rCreateCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.Identifier\x12\x38\n\x10\x63reate_arguments\x18\x02 \x01(\x0b\x32\x1e.com.daml.ledger.api.v1.Record\"\xa7\x01\n\x0f\x45xerciseCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.Identifier\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x0e\n\x06\x63hoice\x18\x03 \x01(\t\x12\x36\n\x0f\x63hoice_argument\x18\x04 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value\"\xcc\x01\n\x14\x45xerciseByKeyCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.Identifier\x12\x33\n\x0c\x63ontract_key\x18\x02 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value\x12\x0e\n\x06\x63hoice\x18\x03 \x01(\t\x12\x36\n\x0f\x63hoice_argument\x18\x04 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value\"\xd5\x01\n\x18\x43reateAndExerciseCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.Identifier\x12\x38\n\x10\x63reate_arguments\x18\x02 \x01(\x0b\x32\x1e.com.daml.ledger.api.v1.Record\x12\x0e\n\x06\x63hoice\x18\x03 \x01(\t\x12\x36\n\x0f\x63hoice_argument\x18\x04 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.ValueBE\n\x16\x63om.daml.ledger.api.v1B\x12\x43ommandsOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3'
  ,
  dependencies=[com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,])




_COMMANDS = _descriptor.Descriptor(
  name='Commands',
  full_name='com.daml.ledger.api.v1.Commands',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ledger_id', full_name='com.daml.ledger.api.v1.Commands.ledger_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_id', full_name='com.daml.ledger.api.v1.Commands.workflow_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='application_id', full_name='com.daml.ledger.api.v1.Commands.application_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='command_id', full_name='com.daml.ledger.api.v1.Commands.command_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='party', full_name='com.daml.ledger.api.v1.Commands.party', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='commands', full_name='com.daml.ledger.api.v1.Commands.commands', index=5,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deduplication_time', full_name='com.daml.ledger.api.v1.Commands.deduplication_time', index=6,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_ledger_time_abs', full_name='com.daml.ledger.api.v1.Commands.min_ledger_time_abs', index=7,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_ledger_time_rel', full_name='com.daml.ledger.api.v1.Commands.min_ledger_time_rel', index=8,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='act_as', full_name='com.daml.ledger.api.v1.Commands.act_as', index=9,
      number=12, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='read_as', full_name='com.daml.ledger.api.v1.Commands.read_as', index=10,
      number=13, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=167,
  serialized_end=540,
)


_COMMAND = _descriptor.Descriptor(
  name='Command',
  full_name='com.daml.ledger.api.v1.Command',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='create', full_name='com.daml.ledger.api.v1.Command.create', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exercise', full_name='com.daml.ledger.api.v1.Command.exercise', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exerciseByKey', full_name='com.daml.ledger.api.v1.Command.exerciseByKey', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='createAndExercise', full_name='com.daml.ledger.api.v1.Command.createAndExercise', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='command', full_name='com.daml.ledger.api.v1.Command.command',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=543,
  serialized_end=831,
)


_CREATECOMMAND = _descriptor.Descriptor(
  name='CreateCommand',
  full_name='com.daml.ledger.api.v1.CreateCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template_id', full_name='com.daml.ledger.api.v1.CreateCommand.template_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='create_arguments', full_name='com.daml.ledger.api.v1.CreateCommand.create_arguments', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=834,
  serialized_end=964,
)


_EXERCISECOMMAND = _descriptor.Descriptor(
  name='ExerciseCommand',
  full_name='com.daml.ledger.api.v1.ExerciseCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template_id', full_name='com.daml.ledger.api.v1.ExerciseCommand.template_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contract_id', full_name='com.daml.ledger.api.v1.ExerciseCommand.contract_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice', full_name='com.daml.ledger.api.v1.ExerciseCommand.choice', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice_argument', full_name='com.daml.ledger.api.v1.ExerciseCommand.choice_argument', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=967,
  serialized_end=1134,
)


_EXERCISEBYKEYCOMMAND = _descriptor.Descriptor(
  name='ExerciseByKeyCommand',
  full_name='com.daml.ledger.api.v1.ExerciseByKeyCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template_id', full_name='com.daml.ledger.api.v1.ExerciseByKeyCommand.template_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contract_key', full_name='com.daml.ledger.api.v1.ExerciseByKeyCommand.contract_key', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice', full_name='com.daml.ledger.api.v1.ExerciseByKeyCommand.choice', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice_argument', full_name='com.daml.ledger.api.v1.ExerciseByKeyCommand.choice_argument', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1137,
  serialized_end=1341,
)


_CREATEANDEXERCISECOMMAND = _descriptor.Descriptor(
  name='CreateAndExerciseCommand',
  full_name='com.daml.ledger.api.v1.CreateAndExerciseCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template_id', full_name='com.daml.ledger.api.v1.CreateAndExerciseCommand.template_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='create_arguments', full_name='com.daml.ledger.api.v1.CreateAndExerciseCommand.create_arguments', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice', full_name='com.daml.ledger.api.v1.CreateAndExerciseCommand.choice', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice_argument', full_name='com.daml.ledger.api.v1.CreateAndExerciseCommand.choice_argument', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1344,
  serialized_end=1557,
)

_COMMANDS.fields_by_name['commands'].message_type = _COMMAND
_COMMANDS.fields_by_name['deduplication_time'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_COMMANDS.fields_by_name['min_ledger_time_abs'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_COMMANDS.fields_by_name['min_ledger_time_rel'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_COMMAND.fields_by_name['create'].message_type = _CREATECOMMAND
_COMMAND.fields_by_name['exercise'].message_type = _EXERCISECOMMAND
_COMMAND.fields_by_name['exerciseByKey'].message_type = _EXERCISEBYKEYCOMMAND
_COMMAND.fields_by_name['createAndExercise'].message_type = _CREATEANDEXERCISECOMMAND
_COMMAND.oneofs_by_name['command'].fields.append(
  _COMMAND.fields_by_name['create'])
_COMMAND.fields_by_name['create'].containing_oneof = _COMMAND.oneofs_by_name['command']
_COMMAND.oneofs_by_name['command'].fields.append(
  _COMMAND.fields_by_name['exercise'])
_COMMAND.fields_by_name['exercise'].containing_oneof = _COMMAND.oneofs_by_name['command']
_COMMAND.oneofs_by_name['command'].fields.append(
  _COMMAND.fields_by_name['exerciseByKey'])
_COMMAND.fields_by_name['exerciseByKey'].containing_oneof = _COMMAND.oneofs_by_name['command']
_COMMAND.oneofs_by_name['command'].fields.append(
  _COMMAND.fields_by_name['createAndExercise'])
_COMMAND.fields_by_name['createAndExercise'].containing_oneof = _COMMAND.oneofs_by_name['command']
_CREATECOMMAND.fields_by_name['template_id'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_CREATECOMMAND.fields_by_name['create_arguments'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._RECORD
_EXERCISECOMMAND.fields_by_name['template_id'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_EXERCISECOMMAND.fields_by_name['choice_argument'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_EXERCISEBYKEYCOMMAND.fields_by_name['template_id'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_EXERCISEBYKEYCOMMAND.fields_by_name['contract_key'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_EXERCISEBYKEYCOMMAND.fields_by_name['choice_argument'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_CREATEANDEXERCISECOMMAND.fields_by_name['template_id'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_CREATEANDEXERCISECOMMAND.fields_by_name['create_arguments'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._RECORD
_CREATEANDEXERCISECOMMAND.fields_by_name['choice_argument'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
DESCRIPTOR.message_types_by_name['Commands'] = _COMMANDS
DESCRIPTOR.message_types_by_name['Command'] = _COMMAND
DESCRIPTOR.message_types_by_name['CreateCommand'] = _CREATECOMMAND
DESCRIPTOR.message_types_by_name['ExerciseCommand'] = _EXERCISECOMMAND
DESCRIPTOR.message_types_by_name['ExerciseByKeyCommand'] = _EXERCISEBYKEYCOMMAND
DESCRIPTOR.message_types_by_name['CreateAndExerciseCommand'] = _CREATEANDEXERCISECOMMAND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Commands = _reflection.GeneratedProtocolMessageType('Commands', (_message.Message,), {
  'DESCRIPTOR' : _COMMANDS,
  '__module__' : 'com.daml.ledger.api.v1.commands_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.Commands)
  })
_sym_db.RegisterMessage(Commands)

Command = _reflection.GeneratedProtocolMessageType('Command', (_message.Message,), {
  'DESCRIPTOR' : _COMMAND,
  '__module__' : 'com.daml.ledger.api.v1.commands_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.Command)
  })
_sym_db.RegisterMessage(Command)

CreateCommand = _reflection.GeneratedProtocolMessageType('CreateCommand', (_message.Message,), {
  'DESCRIPTOR' : _CREATECOMMAND,
  '__module__' : 'com.daml.ledger.api.v1.commands_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.CreateCommand)
  })
_sym_db.RegisterMessage(CreateCommand)

ExerciseCommand = _reflection.GeneratedProtocolMessageType('ExerciseCommand', (_message.Message,), {
  'DESCRIPTOR' : _EXERCISECOMMAND,
  '__module__' : 'com.daml.ledger.api.v1.commands_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ExerciseCommand)
  })
_sym_db.RegisterMessage(ExerciseCommand)

ExerciseByKeyCommand = _reflection.GeneratedProtocolMessageType('ExerciseByKeyCommand', (_message.Message,), {
  'DESCRIPTOR' : _EXERCISEBYKEYCOMMAND,
  '__module__' : 'com.daml.ledger.api.v1.commands_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ExerciseByKeyCommand)
  })
_sym_db.RegisterMessage(ExerciseByKeyCommand)

CreateAndExerciseCommand = _reflection.GeneratedProtocolMessageType('CreateAndExerciseCommand', (_message.Message,), {
  'DESCRIPTOR' : _CREATEANDEXERCISECOMMAND,
  '__module__' : 'com.daml.ledger.api.v1.commands_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.CreateAndExerciseCommand)
  })
_sym_db.RegisterMessage(CreateAndExerciseCommand)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains utilities required by :mod:`dazl.client.config`.
"""

from argparse import _ActionsContainer
from dataclasses import Field, dataclass, field, fields
import logging
from typing import Any, Callable, FrozenSet, Optional, Sequence, Tuple

from ..model.core import Party


def config_field(
    description: str,
    param_type: "Optional[ConfigParameterType]" = None,
    *,
    default_value: Any = None,
    long_alias: Optional[str] = None,
    short_alias: Optional[str] = None,
    deprecated_alias: Optional[str] = None,
    environment_variable: Optional[str] = None
) -> Field:
    return field(
        default=default_value,
        metadata={
            "dazl.config": ConfigParameter(
                description=description,
                param_type=param_type,
                default_value=default_value,
                long_aliases=frozenset([long_alias]) if long_alias else frozenset(),
                short_aliases=frozenset([short_alias]) if short_alias else frozenset(),
                deprecated_aliases=frozenset([deprecated_alias])
                if deprecated_alias
                else frozenset(),
                environment_variable=environment_variable,
            )
        },
    )


def config_fields(class_or_instance: Any) -> "Sequence[Tuple[Field, ConfigParameter]]":
    return [(fld, fld.metadata.get("dazl.config")) for fld in fields(class_or_instance)]


@dataclass(frozen=True)
class ConfigParameter:
    description: str
    param_type: "Optional[ConfigParameterType]" = None
    default_value: Any = None
    long_aliases: FrozenSet[str] = field(default_factory=frozenset)
    short_aliases: FrozenSet[str] = field(default_factory=frozenset)
    deprecated_aliases: FrozenSet[str] = field(default_factory=frozenset)
    environment_variable: Optional[str] = None


@dataclass(frozen=True)
class ConfigParameterType:
    metavar: Optional[str]
    value_ctor: Callable[[Any], Any]


def add_argument(parser: "_ActionsContainer", key: str, config_param: "ConfigParameter") -> None:
    if config_param.param_type is not None:
        aliases = ["--" + key.replace("_", "-")]
        for alias in config_param.short_aliases:
            aliases.append("-" + alias)
        for alias in config_param.long_aliases:
            aliases.append("--" + alias.replace("_", "-"))
        for alias in config_param.deprecated_aliases:
            aliases.append("--" + alias.replace("_", "-"))

        if config_param.param_type.value_ctor == bool:
            parser.add_argument(*aliases, help=config_param.description, action="store_true")
        else:
            default_value = None
            if config_param.environment_variable is not None:
                import os

                default_value = os.getenv(config_param.environment_variable)
            if not default_value:
                default_value = config_param.default_value

            parser.add_argument(
                *aliases,
                type=config_param.param_type.value_ctor,
                metavar=config_param.param_type.metavar,
                help=config_param.description,
                default=default_value
            )


def _parse_url(obj):
    return obj


def _parse_log_level(obj) -> int:
    if not obj:
        return logging.WARNING
    if isinstance(obj, str):
        obj = obj.upper()
    p = logging.getLevelName(obj)
    return p


def _parse_str(obj: Any) -> Optional[str]:
    return str(obj) if obj is not None else None


def _parse_block_offset(obj):
    return obj


def _parse_party_list(obj) -> "Sequence[Party]":
    if isinstance(obj, str):
        return [Party(p) for p in obj.split(",")]
    else:
        return obj


def _parse_package_id_list(obj) -> "Sequence[str]":
    if isinstance(obj, str):
        return obj.split(",")
    else:
        return obj


BOOLEAN_TYPE = ConfigParameterType(None, bool)
COUNT_TYPE = ConfigParameterType("COUNT", int)
BLOCK_OFFSET_TYPE = ConfigParameterType("OFFSET", _parse_block_offset)
LOG_LEVEL_TYPE = ConfigParameterType("LOG_LEVEL", _parse_log_level)
SECONDS_TYPE = ConfigParameterType("SECONDS", float)
URL_TYPE = ConfigParameterType("URL", _parse_url)
PARTIES_TYPE = ConfigParameterType("PARTIES", _parse_party_list)
PACKAGE_IDS_TYPE = ConfigParameterType("PACKAGE_IDS_TYPE", _parse_package_id_list)
PORT_TYPE = ConfigParameterType("PORT", int)
STRING_TYPE = ConfigParameterType("STRING", _parse_str)
PATH_TYPE = ConfigParameterType("PATH", _parse_str)
VERIFY_SSL_TYPE = ConfigParameterType("VERIFY_SSL", _parse_str)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from collections import defaultdict
from dataclasses import dataclass
from functools import reduce
from operator import add
from threading import RLock
from types import MappingProxyType
from typing import Any, Collection, Dict, Generic, Iterable, List, Mapping, Optional, TypeVar, Union
import warnings

from ..damlast.daml_lf_1 import Archive, Expr, Package, PackageRef, TypeConName, ValName, _Name
from ..damlast.lookup import validate_template
from ..util.typing import safe_cast, safe_dict_cast

K = TypeVar("K", bound=_Name)
T = TypeVar("T")

warnings.warn(
    "The types of dazl.model.types_store are deprecated", DeprecationWarning, stacklevel=2
)

with warnings.catch_warnings():
    warnings.simplefilter("ignore", DeprecationWarning)

    from .types import (
        ConcreteType,
        PackageIdSet,
        Template,
        TemplateChoice,
        Type,
        TypeReference,
        UnresolvedTypeReference,
    )


class PackageStoreBuilder:
    """
    Convenience class for building up a :class:`PackageStore`.
    """

    def __init__(self):
        warnings.warn(
            "PackageStoreBuilder is deprecated; there is no direct replacement.",
            DeprecationWarning,
            stacklevel=2,
        )

        self._archives = list()  # type: List[Archive]
        self._value_types = dict()  # type: Dict[ValName, Expr]
        self._data_types = dict()  # type: Dict[TypeConName, Type]
        self._templates = dict()  # type: Dict[TypeConName, Template]
        self._expected_package_ids = None  # type: Optional[Collection[str]]

    def add_archive(self, archive: "Archive") -> None:
        self._archives.append(archive)

    def add_type(self, name: "TypeConName", data_type: Type):
        safe_cast(TypeConName, name)
        self._data_types[name] = safe_cast(Type, data_type)

    def add_value(self, name: "ValName", value: "Expr"):
        safe_cast(ValName, name)
        self._value_types[name] = safe_cast(Expr, value)

    def add_template(self, template: Template):
        self._templates[template.data_type.name.con] = safe_cast(Template, template)

    def add_expected_package_ids(self, expected_package_ids: "Collection[str]"):
        self._expected_package_ids = expected_package_ids

    def get_type(self, name: TypeConName) -> "Optional[Type]":
        return self._data_types.get(name)

    def build(self) -> "PackageStore":
        return PackageStore(
            self._archives,
            self._value_types,
            self._data_types,
            self._templates,
            self._expected_package_ids,
        )


class PackageStore:
    """
    A thread-safe store of type information.
    """

    @classmethod
    def empty(cls):
        """
        Create an empty store.
        """
        warnings.warn(
            "PackageStore is deprecated; use PackageLoader/SymbolLookup instead",
            DeprecationWarning,
            stacklevel=2,
        )

        return cls([], {}, {}, {}, None)

    def __init__(
        self,
        archives: "Collection[Archive]",
        value_types: "Dict[ValName, Expr]",
        data_types: "Dict[TypeConName, Type]",
        templates: "Dict[TypeConName, Template]",
        expected_package_ids: "Optional[Collection[str]]" = None,
    ):
        warnings.warn(
            "PackageStore is deprecated; use PackageLoader/SymbolLookup instead",
            DeprecationWarning,
            stacklevel=2,
        )
        self._lock = RLock()
        self._archives = list(archives)
        self._cache = PackageStoreCache(EMPTY_TYPE_CACHE, EMPTY_TYPE_CACHE, EMPTY_TYPE_CACHE)
        self._value_types = safe_dict_cast(ValName, Expr, value_types)
        self._data_types = safe_dict_cast(TypeConName, Type, data_types)
        self._templates = safe_dict_cast(TypeConName, Template, templates)
        self._expected_package_ids = expected_package_ids

    def archives(self) -> "Collection[Archive]":
        """
        Return a copy of the collection of the set of loaded :class:`Archive`s.
        """
        warnings.warn(
            "PackageStore.archives() is deprecated; use SymbolLookup.archives() instead",
            DeprecationWarning,
            stacklevel=2,
        )

        with self._lock:
            return list(self._archives)

    def packages(self) -> "Collection[Package]":
        """
        Return a copy of the collection of the set of loaded :class:`Package`s.
        """
        warnings.warn(
            "PackageStore.packages() is deprecated; use SymbolLookup.archives() instead",
            DeprecationWarning,
            stacklevel=2,
        )

        with self._lock:
            return [a.package for a in self._archives]

    def package_ids(self) -> "Collection[PackageRef]":
        """
        Return a copy of the collection of the set of loaded :class:`Package`s.
        """
        warnings.warn(
            "PackageStore.package_ids() is deprecated; use SymbolLookup.package_ids() instead",
            DeprecationWarning,
            stacklevel=2,
        )

        with self._lock:
            return [a.hash for a in self._archives]

    def expected_package_ids(self) -> "Collection[str]":
        """
        Return package IDs that are expected to be found on the ledger.
        """
        warnings.warn(
            "PackageStore.expected_package_ids() is deprecated; there is no replacement.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self._expected_package_ids

    def register_all(self, other_store: "PackageStore") -> "PackageStore":
        """
        Register all types declared in the other :class:`PackageStore`.

        :param other_store: A package store to copy types, templates, and choices from.
        :return: A reference to this object.
        """
        warnings.warn(
            "PackageStore.register_all() is deprecated; use PackageLoader instead.",
            DeprecationWarning,
            stacklevel=2,
        )

        if self is not other_store:
            with self._lock:
                self._archives.extend(other_store._archives)
                self._value_types.update(other_store._value_types)
                self._data_types.update(other_store._data_types)
                self._templates.update(other_store._templates)
                self._cache = PackageStoreCache(
                    TypeCache.build(self._value_types),
                    TypeCache.build(self._data_types),
                    TypeCache.build(self._templates),
                )
        return self

    def resolve_value_reference(self, value_ref: "ValName") -> "Expr":
        warnings.warn(
            "PackageStore.resolve_value_reference() is deprecated; "
            "use SymbolLookup.value() instead.",
            DeprecationWarning,
            stacklevel=2,
        )

        with self._lock:
            return self._value_types[value_ref]

    def resolve_type_reference(self, template_ref: "Union[TypeConName, TypeReference]") -> Type:
        """
        Resolve a type based on its reference.

        :param template_ref:
            A :class:`TypeReference` that refers to a type.
        :return:
            The :class:`Type` that is referred to by this type reference.
        :raise KeyError:
            If the :class:`TypeReference` does not have a corresponding value in this store.
        """
        warnings.warn(
            "PackageStore.resolve_value_reference() is deprecated; "
            "use SymbolLookup.data_type() instead.",
            DeprecationWarning,
            stacklevel=2,
        )

        if isinstance(template_ref, TypeReference):
            con = template_ref.con
        elif isinstance(template_ref, TypeConName):
            con = template_ref
        else:
            raise ValueError("template_ref must either by a TypeConName or TypeReference")

        with self._lock:
            return self._data_types[con]

    def types(self) -> "Mapping[TypeConName, ConcreteType]":
        warnings.warn(
            'PackageStore.types() is deprecated; use SymbolLookup.data_type("*") instead.',
            DeprecationWarning,
            stacklevel=2,
        )

        with self._lock:
            return MappingProxyType(dict(self._data_types))

    def get_templates_for_packages(
        self, package_ids: "Iterable[PackageRef]"
    ) -> "Collection[Template]":
        """
        Return a collection of :class:`Template` instances from the given set of package IDs.

        :param package_ids:
            The set of package IDs to restrict templates to.
        :return:
            A collection of matching templates, or an empty collection if none match. This method
            never returns ``None``.
        """
        warnings.warn(
            "PackageStore.get_templates_for_packages() is deprecated; "
            'use [SymbolLookup.templates(f"{pkg}:*") for pkg in package_ids] instead.',
            DeprecationWarning,
            stacklevel=2,
        )

        match = []
        for pkg_id in package_ids:
            match.extend(self._cache.templates.lookup(pkg_id, "*"))
        return match

    def resolve_template(
        self, template: "Union[None, str, TypeReference, UnresolvedTypeReference, TypeConName]"
    ) -> Collection[Template]:
        """
        Return a collection of :class:`Template` instances that match the specified template name.

        Some special parameters:
         * If ``"*"`` is passed in, all templates are returned.
         * If ``None`` is passed in, an empty collection is returned.

        :param template:
            A template name or a :class:`TypeReference`.
        :return:
            A collection of matching templates, or an empty collection if none match. This method
            never returns ``None``.
        """
        warnings.warn(
            "PackageStore.resolve_template() is deprecated; use SymbolLookup.template() instead.",
            DeprecationWarning,
            stacklevel=2,
        )

        if isinstance(template, Template):
            # if we were given a Template for some strange reason, just simply return a single-item
            # tuple of that given Template
            return [template]

        package_id, template_name = validate_template(template, allow_deprecated_identifiers=True)
        return self._cache.templates.lookup(package_id, template_name)

    def resolve_template_type(
        self, template: "Union[None, str, TypeReference, UnresolvedTypeReference, TypeConName]"
    ) -> Dict[TypeReference, Type]:
        """
        Return a collection of types that match for the template.

        :param template:
            A template name or a :class:`TypeReference`.
        :return:
            A dictionary of possible matches, or empty if there are no matches. This method never
            returns ``None``.
        """
        warnings.warn(
            "PackageStore.resolve_template() is deprecated; use SymbolLookup.template() instead.",
            DeprecationWarning,
            stacklevel=2,
        )

        with warnings.catch_warnings():
            warnings.simplefilter("ignore", DeprecationWarning)

            return {
                template.data_type.name: template.data_type
                for template in self.resolve_template(template)
            }

    def resolve_choice(self, template: Any, choice: str) -> Dict[TypeReference, TemplateChoice]:
        """
        Return all possible choices for the combination of template identifier and choice name.
        If template is ``'*'`` or ``None``, all choices with the specified name are returned.
        """
        warnings.warn(
            'PackageStore.resolve_choice() is deprecated; use SymbolLookup.template("*") instead, '
            "and lookup the choice within the returned Template.",
            DeprecationWarning,
            stacklevel=2,
        )

        with warnings.catch_warnings():
            warnings.simplefilter("ignore", DeprecationWarning)

            matches = dict()
            for t in self.resolve_template(template):
                for c in t.choices:
                    if c.name == choice:
                        matches[t.data_type.name] = c
            return matches


class PackageProvider:
    """
    Interface to an object that can provide package information.
    """

    def __init__(self):
        warnings.warn(
            "PackageProvider is deprecated; use PackageLookup instead.",
            DeprecationWarning,
            stacklevel=2,
        )

    def get_package_ids(self) -> "PackageIdSet":
        """
        Return the current universe of package IDs.
        """
        raise NotImplementedError

    def fetch_package(self, package_id: "PackageRef") -> bytes:
        """
        Retrieve the bytes that correspond to a package.
        """
        raise NotImplementedError

    def get_all_packages(self) -> "Mapping[PackageRef, bytes]":
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", DeprecationWarning)
            return {pkg_id: self.fetch_package(pkg_id) for pkg_id in self.get_package_ids()}


class MemoryPackageProvider(PackageProvider):
    def __init__(self, mapping: "Mapping[PackageRef, bytes]"):
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", DeprecationWarning)
            super().__init__()
        warnings.warn(
            "MemoryPackageProvider is deprecated; there is no replacement.",
            DeprecationWarning,
            stacklevel=2,
        )
        self.mapping = mapping

    def get_package_ids(self) -> "PackageIdSet":
        return frozenset(self.mapping.keys())

    def fetch_package(self, package_id: "PackageRef") -> bytes:
        return self.mapping.get(package_id)


@dataclass(frozen=True)
class PackageStoreCache:
    value_types: "TypeCache[ValName, Expr]"
    data_types: "TypeCache[TypeConName, Type]"
    templates: "TypeCache[TypeConName, Template]"


@dataclass(frozen=True)
class TypeCache(Generic[K, T]):
    """
    Immutable cache of type information.

    Instance attributes:

    .. attribute:: TypeCache.by_package_lookup

        A mapping from package ID strings to another mapping, where keys are fully-qualified type
        names for templates to a collection of matching Template instances. These collections are
        never non-empty; they will only ever contain multiple entries if there is a collision in
        dot-separated names.

    .. attribute:: TypeCache.by_name_lookup

        A mapping from fully-qualified type names (excluding package IDs) to ``Collection[T]``.
        These subcollections will only contain multiple values if names are not unique with respect
        to package IDs.
    """

    everything: "Collection[T]"
    by_package_lookup: "Mapping[str, Mapping[str, Collection[T]]]"
    by_name_lookup: "Mapping[str, Collection[T]]"

    @classmethod
    def build(cls, objects: "Mapping[K, T]") -> "TypeCache[K, T]":
        from ..damlast.util import package_local_name, package_ref

        everything = tuple(objects.values())
        by_package_lookup = defaultdict(lambda: defaultdict(list))
        by_name_lookup = defaultdict(list)

        for k, v in objects.items():
            package_id = package_ref(k)
            module_entity_name = package_local_name(k)
            # This is kept here for backwards compatibility, but its use should be discouraged
            module_entity_name_deprecated = module_entity_name.replace(":", ".")
            for valid_name in (module_entity_name, module_entity_name_deprecated):
                by_package_lookup[package_id][valid_name].append(v)
                by_name_lookup[valid_name].append(v)

        return TypeCache(
            everything,
            _immutable_mmc(by_package_lookup),
            MappingProxyType({k: tuple(v) for k, v in by_name_lookup.items()}),
        )

    def lookup(self, package_id: str, type_name: str) -> "Collection[T]":
        """
        Look up items based on package ID and type name. The values cannot be ``None``, but they
        can be the special-meaning ``'*'`` value.
        """
        safe_cast(str, package_id)
        safe_cast(str, type_name)

        if package_id == "*":
            if type_name == "*":
                # *:* means return everything
                return self.everything
            else:
                # *:Specific.Module:Entity means return all matches of the requested name, but
                # under any package
                return self.by_name_lookup.get(type_name, ())
        elif type_name == "*":
            # PKG_ID:* means return all matches in a specific package
            return reduce(add, self.by_package_lookup.get(package_id, {}).values(), ())
        else:
            # PKG_ID:Specific.Module:Entity; we are looking for a very specific type
            return self.by_package_lookup.get(package_id, {}).get(type_name, ())


def _immutable_mmc(
    mapping: "Mapping[str, Mapping[str, Collection[T]]]",
) -> "Mapping[str, Mapping[str, Collection[T]]]":
    """
    Create an immutable copy of :class:`TemplateStoreCache` data structures.
    """
    return MappingProxyType(
        {k1: MappingProxyType({k2: tuple(v) for k2, v in v1.items()}) for k1, v1 in mapping.items()}
    )


EMPTY_MAPPING = MappingProxyType({})
EMPTY_TYPE_CACHE = TypeCache((), EMPTY_MAPPING, EMPTY_MAPPING)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from os import PathLike
from pathlib import Path
from typing import Union


def pathify(path: "Union[str, Path, PathLike]") -> Path:
    """
    Convert an object that could be a :class:`Path` into a :class:`Path`.
    """
    if isinstance(path, Path):
        return path
    elif isinstance(path, PathLike):
        return Path(path)
    else:
        raise ValueError(f"path must be a str or Path (got {path!r} instead)")



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
:mod:`dazl.pretty` package
==========================

This module contains utilities for pretty-printing various types in dazl.

.. automodule:: dazl.pretty.render_daml
.. automodule:: dazl.pretty.util
"""

from typing import TYPE_CHECKING, Optional, Type

from ..model.types_store import PackageStore
from ._render_base import PrettyPrintBase, pretty_print_syntax
from .options import PrettyOptions
from .render_csharp import CSharpPrettyPrint
from .render_daml import DEFAULT_PRINTER as DAML_PRETTY_PRINTER, DamlPrettyPrinter
from .render_python import PythonPrettyPrint
from .util import maybe_parentheses

if TYPE_CHECKING:
    from .pygments_daml_lexer import DAMLLexer as _DAMLLexer_TYPE


def _import_daml_lexer() -> "Optional[Type[_DAMLLexer_TYPE]]":
    # pygments isn't absolutely required, but if it's loaded, also provide our lexer
    try:
        from .pygments_daml_lexer import DAMLLexer

        return DAMLLexer
    except ImportError:
        return None


DAMLLexer = _import_daml_lexer()


ALL_PRINTER_TYPES = [CSharpPrettyPrint, DamlPrettyPrinter, PythonPrettyPrint]


# noinspection PyShadowingBuiltins,PyShadowingNames
def get_pretty_printer(
    format: str, options: "PrettyOptions", store: "PackageStore"
) -> "Optional[PrettyPrintBase]":
    for printer in ALL_PRINTER_TYPES:
        if printer.syntax.startswith(format):
            return printer(store, options)
    return None



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Define a few DAML values and functions that have been inlined in Python for efficiency and
simplicity.
"""

from typing import Any, Optional, Sequence

from . import daml_types as daml
from ._builtins_meta import Builtin, BuiltinTable
from .daml_lf_1 import BuiltinFunction, Expr, PrimLit, Type

builtins = BuiltinTable()


@builtins.add
class Concat(Builtin):
    name = "DA.Internal.Prelude:concat"
    args = (daml.List(daml.List(daml.var("x"))),)

    def evaluate(self, _: "Sequence[Type]", args: "Sequence[Any]") -> "Any":
        xxs = args[0]

        result = []
        for xs in xxs:
            result.extend(xs)
        return result

    def simplify(self, _: "Sequence[Type]", args: "Sequence[Expr]") -> "Optional[Expr]":
        xxs = args[0]

        # if there is a single-element list as an arg to concat, simply remove the function call
        # and fully unwrap
        if xxs.cons is not None and xxs.cons.tail.nil is not None and len(xxs.cons.front) == 1:
            return xxs.cons.front[0]
        if xxs.nil is not None:
            return Expr(nil=Expr.Nil(xxs.nil.type.prim.args[0]))


@builtins.add
class Show(Builtin):
    name = "GHC.Show:show"
    args = (daml.Text, daml.Text)

    def evaluate(self, _: "Sequence[Type]", args: "Sequence[Any]") -> "Any":
        return str(args[1])

    def simplify(self, _: "Sequence[Type]", args: "Sequence[Expr]") -> "Optional[Expr]":
        return None


@builtins.add
class AppendText(Builtin):
    builtin = BuiltinFunction.APPEND_TEXT
    args = (daml.Text, daml.Text)

    def evaluate(self, _: "Sequence[Type]", args: "Sequence[Any]") -> "Any":
        return args[0] + args[1]

    def simplify(self, _: "Sequence[Type]", args: "Sequence[Expr]") -> "Expr":
        if args[0].prim_lit is not None and args[1].prim_lit is not None:
            return Expr(prim_lit=PrimLit(text=args[0].prim_lit.text + args[1].prim_lit.text))



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
from argparse import ArgumentParser


class CliCommand:
    @property
    def name(self) -> str:
        raise NotImplementedError("command needs a name")

    @property
    def hidden(self) -> bool:
        return False

    def parser(self) -> ArgumentParser:
        raise NotImplementedError("ArgumentParser not supplied")

    def execute(self, args) -> int:
        raise NotImplementedError("Command not implemented")



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Methods for serializing domain objects and other things into basic primitive
types over the wire on the REST interface using JSON.
"""

from datetime import datetime
from typing import Any
import warnings

from ...damlast.daml_lf_1 import TypeConName
from ...model.writing import AbstractSerializer, CommandPayload
from ...prim import ContractId, JSONEncoder
from ...values.json import JsonEncoder


class LedgerJSONEncoder(JSONEncoder):
    """
    Convert some known Ledger API primitive types into their appropriate JSON
    representations.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        warnings.warn(
            "dazl.protocols.v0.json_ser_command.LedgerJSONEncoder is deprecated; "
            "use dazl.prim.JSONEncoder instead.",
            DeprecationWarning,
            stacklevel=2,
        )


def to_api_datetime(obj):
    """
    Converts the object to an ISO8601 datetime string.

    :param obj:
        A datetime. If the datetime is "naive" (no timezone information), it is
        assumed to refer to UTC. If the datetime has timezone information, the
        datetime will be converted to UTC before serializing.
    :return: An ISO8601 string that represents the time in UTC.
    """
    from ...prim import datetime_to_str

    warnings.warn(
        "dazl.protocols.v0.json_ser_command.to_api_datetime is deprecated; "
        "use dazl.prim.datetime_to_str instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return datetime_to_str(obj) if isinstance(obj, datetime) else obj


class JsonSerializer(AbstractSerializer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        warnings.warn(
            "dazl.protocols.v0.json_ser_command.JsonSerializer is deprecated; "
            "there is no replacement.",
            DeprecationWarning,
            stacklevel=2,
        )

    mapper = JsonEncoder()

    def serialize_command_request(self, command_payload: CommandPayload) -> dict:
        commands = [self.serialize_command(command) for command in command_payload.commands]
        return dict(
            businessIntent=command_payload.command_id,
            commands=commands,
            application=command_payload.application_id,
        )

    def serialize_create_command(self, template_name: "TypeConName", template_args: "Any") -> "Any":
        # the package_id in ModuleRef is a convenient place to
        # stash legacy template IDs in the REST endpoint
        return {"create": {"template": str(template_name), "arguments": template_args}}

    def serialize_exercise_command(
        self, contract_id: ContractId, choice_name: str, choice_args: "Any"
    ) -> "Any":
        return {
            "exercise": {
                "contract": contract_id.value,
                "choice": choice_name,
                "arguments": choice_args,
            }
        }



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import command_submission_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__submission__service__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class CommandSubmissionServiceStub(object):
    """Allows clients to attempt advancing the ledger's state by submitting commands.
    The final states of their submissions are disclosed by the Command Completion Service.
    The on-ledger effects of their submissions are disclosed by the Transaction Service.

    Commands may fail in 2 distinct manners:

    1. Failure communicated in the gRPC error of the submission.
    2. Failure communicated in a Completion.

    Only successfully submitted commands may produce a completion event.

    Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
    Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

    Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
    The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Submit = channel.unary_unary(
                '/com.daml.ledger.api.v1.CommandSubmissionService/Submit',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__submission__service__pb2.SubmitRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )


class CommandSubmissionServiceServicer(object):
    """Allows clients to attempt advancing the ledger's state by submitting commands.
    The final states of their submissions are disclosed by the Command Completion Service.
    The on-ledger effects of their submissions are disclosed by the Transaction Service.

    Commands may fail in 2 distinct manners:

    1. Failure communicated in the gRPC error of the submission.
    2. Failure communicated in a Completion.

    Only successfully submitted commands may produce a completion event.

    Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
    Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

    Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
    The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
    """

    def Submit(self, request, context):
        """Submit a single composite command.
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        - ``NOT_FOUND``: if the request does not include a valid ledger id
        - ``INVALID_ARGUMENT``: if the payload is malformed or is missing required fields
        - ``UNAVAILABLE``: if the participant is not yet ready to submit commands or if the service has been shut down.
        - ``RESOURCE_EXHAUSTED``: if the participant or the ledger is overloaded. Clients should back off exponentially and retry.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CommandSubmissionServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Submit': grpc.unary_unary_rpc_method_handler(
                    servicer.Submit,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__submission__service__pb2.SubmitRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.daml.ledger.api.v1.CommandSubmissionService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CommandSubmissionService(object):
    """Allows clients to attempt advancing the ledger's state by submitting commands.
    The final states of their submissions are disclosed by the Command Completion Service.
    The on-ledger effects of their submissions are disclosed by the Transaction Service.

    Commands may fail in 2 distinct manners:

    1. Failure communicated in the gRPC error of the submission.
    2. Failure communicated in a Completion.

    Only successfully submitted commands may produce a completion event.

    Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
    Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

    Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
    The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
    """

    @staticmethod
    def Submit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.CommandSubmissionService/Submit',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__submission__service__pb2.SubmitRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Contains primitive declarations and functions for working with "native" Python types as they
correspond to types over the Ledger API.
"""

from .basic import to_bool, to_str
from .complex import to_record, to_variant
from .contracts import ContractData, ContractId
from .datetime import (
    TimeDeltaLike,
    date_to_int,
    date_to_str,
    datetime_to_epoch_microseconds,
    datetime_to_epoch_timedelta,
    datetime_to_str,
    datetime_to_timestamp,
    timedelta_to_duration,
    to_date,
    to_datetime,
    to_timedelta,
)
from .json import JSONEncoder
from .map import FrozenDict
from .numbers import decimal_to_str, to_decimal, to_int
from .party import Party, to_party



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
:mod:`dazl.damast` package
==========================

The :mod:`dazl.damlast` module contains types and functions for working with DAML-LF Archives.
The types in this module are for dealing with DAML's type system as encoded in DAML-LF; for
encoding and decoding of values, see :mod:`dazl.values`.

:mod:`dazl.damlast.daml_lf_1`:
    The full definition of a DAML-LF Archive.

:mod:`dazl.damlast.daml_types`:
    Convenience functions for constructing DAML :class:`Type` objects.

:mod:`dazl.damlast.lookup`:
    Utilities for quickly resolving names to DAML-LF types/values.

:mod:`dazl.damlast.parse`:
    Functions for parsing a DAML-LF Archive from its Protobuf definition.

:mod:`dazl.damlast.protocols`:
    Protocols (interfaces) for components in this package.

:mod:`dazl.damlast.errors`:
    Subclasses of :class:`Error` that may be thrown by classes in this package.

.. automodule:: dazl.damlast.daml_lf_1

    :members:
.. automodule:: dazl.damlast.daml_types
    :members:
.. automodule:: dazl.damlast.lookup
    :members:
.. automodule:: dazl.damlast.parse
    :members:
.. automodule:: dazl.damlast.errors
.. automodule:: dazl.damlast.protocols

"""

from .pkgfile import CachedDarFile, DarFile, get_dar_package_ids
from .visitor import ExprVisitor, IdentityTypeVisitor, ModuleVisitor, PackageVisitor, TypeVisitor



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Mapping, Optional, Sequence, Union
import warnings

from .daml_lf_1 import (
    UNIT,
    DefDataType,
    DefValue,
    DottedName,
    Expr,
    Kind,
    ModuleRef,
    PackageRef,
    PrimType,
    Type,
    TypeVarWithKind,
    _Name,
)

if TYPE_CHECKING:
    from ..model.types import Type as OldType, TypeReference
    from ..model.types_store import PackageStore


def var(var: str) -> "Type":
    warnings.warn("dazl.damlast.util.var is deprecated; use dazl.damlast.daml_types.var instead.")
    from .daml_types import var as _var

    return _var(var)


def values_by_module(
    store: "PackageStore",
) -> "Mapping[ModuleRef, Mapping[Sequence[str], Union[Expr, OldType]]]":
    from collections import defaultdict

    d = defaultdict(defaultdict)
    for vn, vv in store._value_types.items():
        d[vn.module][vn.name] = vv
    for vn, vv in store._data_types.items():
        d[vn.module][vn.name] = vv
    return d


# noinspection PyShadowingBuiltins
def unpack_arrow_type(type: "Optional[Type]") -> "Sequence[Type]":
    """
    Unwind an arrow type the sequence of constituent types.

    If presented with something that is not an arrow type, it is simply returned.
    """
    if type is None:
        return []

    types = []
    next = type
    while True:
        if next.forall is not None:
            next = next.forall.body
        if next.prim is not None and next.prim.prim == PrimType.ARROW:
            types.append(next.prim.args[0])
            next = next.prim.args[1]
        else:
            types.append(next)
            return types


# noinspection PyShadowingBuiltins
def pack_arrow_type(types: "Sequence[Type]") -> "Optional[Type]":
    """
    Compute the arrow type that is the result of repeatedly applying values to a function and
    returning it.
    """
    if not types:
        return None

    t = types[-1]
    for type in reversed(types[0:-1]):
        t = arrow_type(type, t)
    return t


# noinspection PyShadowingBuiltins
def arrow_type(input: "Type", output: "Type") -> "Type":
    """
    Convenience function for constructing a :class:`Type` of ``prim`` that is an abstraction taking
    the input type and returning the output type.
    """
    warnings.warn(
        "arrow_type is deprecated; Use dazl.damlast.daml_types.Arrow instead",
        DeprecationWarning,
        stacklevel=2,
    )
    from .daml_types import Arrow

    return Arrow(input, output)


def list_type(elem_type: "Type") -> "Type":
    """
    Convenience function for constructing a :class:`Type` of ``prim`` list.
    """
    warnings.warn(
        "list_type is deprecated; Use dazl.damlast.daml_types.List instead",
        DeprecationWarning,
        stacklevel=2,
    )
    from .daml_types import List

    return List(elem_type)


def type_var_with_kind(var: str, type: Kind = Kind(star=UNIT)):
    return TypeVarWithKind(var, type)


def def_value(name: "Union[str, Sequence[str]]", daml_type: "Type", expr: "Expr") -> "DefValue":
    if isinstance(name, str):
        name = (name,)
    name_with_type = DefValue.NameWithType(name=name, type=daml_type)
    return DefValue(name_with_type, expr, True, False, None)


def package_ref(obj: "Union[ModuleRef, _Name, TypeReference]") -> "PackageRef":
    """
    Return the package ID for a :class:`ModuleRef` or a DAML name.
    """
    from ..model.types import TypeReference

    # TODO: Rewrite for dazl 7.0.0 when the internal structure of a ModuleRef is changed.
    if isinstance(obj, ModuleRef):
        # noinspection PyProtectedMember
        return obj._package_id
    elif isinstance(obj, _Name):
        # noinspection PyProtectedMember
        return obj._module._package_id
    elif isinstance(obj, TypeReference):
        # noinspection PyProtectedMember
        return obj.con._module._package_id
    else:
        raise ValueError(f"Could not extract a package_ref from {obj!r}")


def module_name(obj: "Union[ModuleRef, _Name, TypeReference]") -> "DottedName":
    """
    Return the module name of a :class:`ModuleRef` or a DAML name.
    """
    from ..model.types import TypeReference

    # TODO: Rewrite for dazl 7.0.0 when the internal structure of ModuleRefs and _Name are changed.
    if isinstance(obj, ModuleRef):
        # noinspection PyProtectedMember
        return obj._module_name
    elif isinstance(obj, _Name):
        # noinspection PyProtectedMember
        return obj._module._module_name
    elif isinstance(obj, TypeReference):
        # noinspection PyProtectedMember
        return obj.con._module._module_name
    else:
        raise ValueError(f"Could not extract a module_name from {obj!r}")


def module_ref(obj: "Union[_Name, TypeReference]") -> "ModuleRef":
    """
    Return the :class:`ModuleRef` of a DAML name.
    """
    from ..model.types import TypeReference

    # TODO: Rewrite for dazl 7.0.0 when the internal structure of ModuleRefs and _Name are changed.
    if isinstance(obj, _Name):
        # noinspection PyProtectedMember
        return obj._module
    elif isinstance(obj, TypeReference):
        # noinspection PyProtectedMember
        return obj.con._module
    else:
        raise ValueError(f"Could not extract a module_ref from {obj!r}")


def package_local_name(obj: "Union[_Name, TypeReference]") -> str:
    """
    Return the name of a DAML object, assuming that the referent exists in the same package as the
    target (i.e., _not_ scoped with a package ID).
    """
    from ..model.types import TypeReference

    # TODO: Rewrite for dazl 7.0.0 when the internal structure of a ModuleRef is changed.
    if isinstance(obj, TypeReference):
        obj = obj.con

    if isinstance(obj, _Name):
        # noinspection PyProtectedMember
        return f'{obj._module._module_name}:{".".join(obj._name)}'
    else:
        raise ValueError(f"Could not extract a package_local_name from {obj!r}")


def module_local_name(obj: "Union[_Name, TypeReference]") -> str:
    """
    Return the name of a DAML object, assuming that the referent exists in the same module as the
    target (i.e., in the same package and in the same module).
    """
    # TODO: Rewrite for dazl 7.0.0 when the internal structure of ModuleRefs and _Name are changed.
    from ..model.types import TypeReference

    if isinstance(obj, _Name):
        # noinspection PyProtectedMember
        return ".".join(obj._name)
    elif isinstance(obj, TypeReference):
        # noinspection PyProtectedMember
        return ".".join(obj.con._name)
    else:
        raise ValueError(f"Could not extract a module_local_name from {obj!r}")


def find_variant_type(dt: "DefDataType", variant: "DefDataType.Fields", constructor: str) -> "Type":
    for fld_metadata in variant.fields:
        if fld_metadata.field == constructor:
            return fld_metadata.type

    raise ValueError(f"{constructor} is not a field of {dt.name}")



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

# Type definitions for _scheduler.py.
#
# These type definitions live in a separate file instead of in _scheduler.py because
#  * The overloads for run_in_loop visually pollute the run_in_loop implementation.
#  * The typing module defines Awaitable[T], but the asyncio module defines a non-generic Future.
#    "Properly" typing the signatures of these functions would result in a bit of unnecessary and
#    inaccurate casting in the bodies of these methods.
#    (See https://github.com/python/typing/issues/446 for more information on this.)

from asyncio import AbstractEventLoop, Future
from concurrent.futures import Executor
from typing import Awaitable, Callable, List, Optional, TypeVar, overload

from . import RunLevel
from ..prim import TimeDeltaLike

T = TypeVar("T", covariant=True)

class Invoker:
    level: RunLevel
    loop: Optional[AbstractEventLoop]
    executor: Optional[Executor]
    _futures: List[Future]
    def __init__(self): ...
    async def __aenter__(self): ...
    async def __aexit__(self, exc_type, exc_val, exc_tb): ...
    def create_future(self) -> Future: ...
    def _unhook_future(self, fut: Future) -> None: ...
    async def shutdown(
        self, timeout: "TimeDeltaLike" = ..., exception: "Optional[Exception]" = None
    ) -> None: ...
    def set_context_as_current(self) -> None: ...
    @overload
    def run_in_loop(self, func: Callable[[], Awaitable[None]], timeout: float = 30) -> None: ...
    @overload
    def run_in_loop(self, func: Callable[[], None], timeout: float = 30) -> None: ...
    @overload
    def run_in_loop(self, func: Callable[[], Awaitable[T]], timeout: float = 30) -> T: ...
    @overload
    def run_in_loop(self, func: Callable[[], T], timeout: float = 30) -> T: ...
    def run_in_executor(self, func: "Callable[[], T]") -> Awaitable[T]: ...
    def handle_sigint(self): ...
    def handle_sigquit(self): ...



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from typing import Any

__all__ = ["to_bool", "to_str"]


def to_bool(obj: "Any") -> bool:
    """
    Convert any of the common wire representations of a ``bool`` to a ``bool``.
    """
    if obj is None:
        # noinspection PyTypeChecker
        return None

    if isinstance(obj, bool):
        return obj
    elif isinstance(obj, str):
        lobj = obj.lower().strip()
        if lobj == "true":
            return True
        elif lobj == "false":
            return False
        else:
            raise ValueError(f"Could not parse a string as a boolean: {obj!r}")
    raise ValueError(f"Could not parse as a boolean: {obj!r}")


def to_str(obj: "Any") -> str:
    """
    Convert any object to a string. This simply calls ``str`` on the object to produce a string
    representation.
    """
    return obj if isinstance(obj, str) else str(obj)
